import { CLOUD_SERVICE_PROVIDER_LIST, DEFAULT_CLOUD_CONTEXTS } from 'constant/others/other';
import useDrupalJsonApi from 'hooks/drupal_jsonapi';
import CloudContext from 'model/CloudContext';
import CloudServiceProvider from 'model/CloudServiceProvider';
import { createContext, useEffect, useState } from 'react';

interface Message {
  text: string;
  color: 'primary' | 'secondary' | 'success' | 'danger' | 'warning' | 'info' | 'light' | 'dark';
}

interface AppState {
  cloudContextList: CloudContext[];
  messages: Message[];
  addMessage: (message: string) => void;
  addErrorMessage: (message: string) => void;
  removeMessage: (messageIndex: number) => void;
  loadingFlg: boolean;
}

export const useAppState = (): AppState => {
  const { getEntityListAll } = useDrupalJsonApi();
  const [cloudContextList, setCloudContextList] = useState<CloudContext[]>([...DEFAULT_CLOUD_CONTEXTS]);
  const [messages, setMessages] = useState<Message[]>([]);
  const [loadingFlg, setLoadingFlg] = useState(true);

  // Set cloud context list.
  useEffect(() => {
    const init = async () => {
      let newCloudContextList = [...DEFAULT_CLOUD_CONTEXTS];
      for (const cloudServiceProvider of CLOUD_SERVICE_PROVIDER_LIST) {
        try {
          const data = (await getEntityListAll('cloud_config', {}, cloudServiceProvider))
            .map((record: any) => {
              return {
                cloudServiceProvider: cloudServiceProvider as CloudServiceProvider,
                name: record.attributes.cloud_context,
                labelName: record.attributes.name
              };
            });
          newCloudContextList = [...newCloudContextList, ...data];
        } catch {
          // pass
        }
      }
      setCloudContextList(newCloudContextList);
      setLoadingFlg(false);
    };
    init();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  /**
   * Add message for display.
   * @param message The message.
   */
  const addMessage = (message: string) => {
    setMessages([...messages, {
      text: message,
      color: 'success',
    }]);
  }

  /**
   * Add error message for display.
   * @param message The message.
   */
  const addErrorMessage = (message: string) => {
    setMessages([...messages, {
      text: message,
      color: 'danger',
    }]);
  }

  /**
   * Remove message for display.
   * @param messageIndex The message index.
   */
  const removeMessage = (messageIndex: number) => {
    const newMessages = [...messages];
    newMessages.splice(messageIndex, 1);
    setMessages(newMessages);
  }

  return {
    cloudContextList,
    messages,
    addMessage,
    addErrorMessage,
    removeMessage,
    loadingFlg
  };
};

export const AppContext = createContext<AppState>({
  cloudContextList: [],
  messages: [],
  addMessage: () => {},
  addErrorMessage: () => {},
  removeMessage: () => {},
  loadingFlg: true,
});
